//---------------------------------------------------------------------------
// TConnectorSample.cpp ... (c) 2002   TEC-IT Datenverarbeitung GmbH, A-4400 Steyr
// This source code was made for demonstration purposes only.
// If you want to publish it anyway, please do it only with our permission.
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "TConnectorSample.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "TCONNECTOR2Lib_OCX"
#pragma resource "*.dfm"

TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{

}
//---------------------------------------------------------------------------

void __fastcall TForm1::btnSettingsClick(TObject *Sender)
{
        if (TC2->IsOpen)
                Application->MessageBox("Close connection first!", NULL, MB_OK);
        else
                TC2->PropertyDialog();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::btnOpenClick(TObject *Sender)
{
long eCode = S_OK;

        // OPENING: WHY WE USE THE "CONTROLINTERFACE"?
        // We open the connection by the ControlInterface of our ActiveX
        // The ControlInterface is "inside" of Borlands created ActiveX wrapper,
        // from there we get a return value if the call was successful.

        eCode = TC2->ControlInterface.Open();

        if (eCode == S_OK)
        {
                this->btnOpen->Enabled = false;
        }
        else
        {
               //display more information about the error
                String sErrText = TC2->TranslateErrorNo(eCode);
                Application->MessageBox(sErrText.c_str(), "Error", MB_OK);
        }
}
//---------------------------------------------------------------------------



void __fastcall TForm1::btnCloseClick(TObject *Sender)
{
        TC2->ControlInterface.Close();

        this->btnOpen->Enabled = true;
        this->btnStartListen->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::btnStartListenClick(TObject *Sender)
{
long eCode  = S_OK;
BSTR bstrData;

        if (!TC2->IsOpen)
        {
                Application->MessageBox("Device not open!", "Error...", MB_OK);
                return;
        }

        // Timeout_Infinite = false: the OnData event is fired after a period of time specified in Timeout property
        // Timeout_Infinite = true: the event occurs only after NoOfBytes are received

        TC2->Timeout_Infinite = false;

        TC2->NoOfBytes = 30;               // fire event after 30 Bytes has been received or specified Timeout has been reached

        /* if you want, you can clear the read in buffer before StartListen()
        {
        long nBytes   = 1;
        BSTR bstrData = SysAllocString(NULL);

                eCode = S_OK;
                while ((eCode == S_OK) && (nBytes > 0))
                {
                        eCode = TC2->ControlInterface.Read(65000, 100, &bstrData);
                        nBytes = TC2->GetNoOfBytesRead();
                }
                SysFreeString(bstrData);
        } */

        eCode = TC2->ControlInterface.StartListen();  // enable OnData events

        if (eCode != S_OK)
        {
               //display more information about the error
                String sErrText = TC2->TranslateErrorNo(eCode);
                Application->MessageBox(sErrText.c_str(), "Error", MB_OK);
        }
        else
        {
               this->btnStartListen->Enabled = false;
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::btnStopListenClick(TObject *Sender)
{
long eCode = TC2->ControlInterface.StopListen();    // disable OnData events

        if (eCode == S_OK)
        {
               this->btnStartListen->Enabled = true;
        }
        else
        {
               //display more information about the error
               String sErrText = TC2->TranslateErrorNo(eCode);
               Application->MessageBox(sErrText.c_str(), "Error", MB_OK);
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::TConnOnData(TObject *Sender, BSTR sData)
{
        // if we are in Keystroke Simulation Mode, we don't want any additional action
        if (TC2->SendKeyStrokes)
                return;

        // sData contains the received data as BSTR
        // within one event we reveice a maximum of NoOfBytes Property
        this->lbData->Items->Add(sData);

        // select the last entry of the data list box
        int nCount = this->lbData->Items->Count;
        this->lbData->ItemIndex = nCount - 1; // >ScrollBy(0, 1000); // scroll down
}
//---------------------------------------------------------------------------

void __fastcall TForm1::btnReadClick(TObject *Sender)
{
long nBytes   = 0;
BSTR bstrData = SysAllocString(NULL);   // allocate memory for data string
long eCode    = S_OK;
long nTimeout = (this->lbTimeout->ItemIndex+1)*1000;  // Timeout in msec

        this->btnRead->Enabled = false;

        // we want to read 100 Bytes within the selected Timeout
        eCode = TC2->ControlInterface.Read(100, nTimeout, &bstrData);
        nBytes = TC2->GetNoOfBytesRead();  // get number of bytes actually read

        if (eCode == S_OK)
        {
        TVarRec v[] = { (Integer) nBytes };
        String msg  = Format("%d Bytes read", v, ARRAYSIZE(v)-1);

                Application->MessageBox(msg.c_str(), "Success", MB_OK);
                this->lbDataRead->Items->Add(bstrData);
                this->lbDataRead->ItemIndex = this->lbDataRead->Items->Count - 1; // select the last entry of the data list box
        }
        else
        {
               //display more information about the error
                String sErrText = TC2->TranslateErrorNo(eCode);
                Application->MessageBox(sErrText.c_str(), "Error", MB_OK);
        }

        SysFreeString(bstrData);  // avoid memory leaks

        this->btnRead->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::InitForm(TObject *Sender)
{
        this->lbTimeout->ItemIndex = 2; // select second

        // if there is a TConnector Object available
        // call the license method
        if (this->TC2)
                this->LicenseAtStartup(this->TC2);
        else
                Application->MessageBoxA("Could not license TConnector2!", "Error", MB_OK);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::chkKeyStrokeOnClick(TObject *Sender)
{

        if (chkKeyStroke->Checked)
        {
                // 1. step:  enable key stroke simulation mode before we open the connection !!
                TC2->SendKeyStrokes = true;

                // we want that CR (ASCII = 13) within the input data
                // to press the default button...
                TC2->IncludeDelimiter = true;
                TC2->Delimiter        = "\\x0d";
                TC2->UseDelimiter     = true;

                // if connection is opened, close it
                if (TC2->IsOpen)
                        TC2->Close();

                // 2. step: open connection
                this->btnOpenClick(this);         //  refer to code in "btnOpenClick..."

                // 3. step: enable OnData events (to enable Keystroke Simulation we need the OnData event)
                this->btnStartListenClick(this);

                this->editKeyStroke->SetFocus();  // set focus to field where we want to receive the key strokes
        }
        else
        {
                TC2->SendKeyStrokes = false;
                TC2->UseDelimiter   = false;
                this->btnStopListenClick(this);
                this->btnCloseClick(this);
        }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::btnReturnClick(TObject *Sender)
{
    // add entered data to the list box
    String sInfo = this->editKeyStroke->Text;

        Application->MessageBox("Now adding data to the list box...", "CR hit the button", MB_OK);
        // Note: if there is a LineFeed character behind the Carriage Return,
        // this MessageBox closes automatically


        this->lbKeyStrData->Items->Add(sInfo);
        // select the last entry of the data list box
        int nCount = this->lbKeyStrData->Items->Count;
        this->lbKeyStrData->ItemIndex = nCount - 1;
        this->editKeyStroke->Clear();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::btnWriteClick(TObject *Sender)
{
long    eCode           = S_OK;
long    nTimeout        = (this->lbTimeout->ItemIndex+1)*1000;  // Timeout in msec
long    nBytesWritten   = 0;
long    nBufSize        = edDataToWrite->Text.WideCharBufSize();
long    nBytes          = edDataToWrite->Text.Length();
wchar_t*bstrData        = new wchar_t[ nBytes + 1 ];

        edDataToWrite->Text.WideChar(bstrData, nBufSize);  // convert data to widechar*

        this->btnWrite->Enabled = false;

        // we want to write nBytes within the selected Timeout
        eCode = TC2->ControlInterface.Write(nBytes, nTimeout, bstrData, &nBytesWritten);

        if (eCode == S_OK)
        {
        TVarRec v[] = { (Integer) nBytesWritten, (Integer) nBytes, (Integer) nTimeout };
        String msg  = Format("%d of %d Bytes written (Timeout=%d ms)", v, ARRAYSIZE(v)-1);

                Application->MessageBox(msg.c_str(), "Success", MB_OK);
        }
        else
        {
               //display more information about the error
                String sErrText = TC2->TranslateErrorNo(eCode);
                Application->MessageBox(sErrText.c_str(), "Error", MB_OK);
        }

        delete []bstrData;  // avoid memory leaks

        this->btnWrite->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::LicenseAtStartup(TTConnector2* pTC2)
{
// you can order your license key directly from http://www.tec-it.com/order/
// the key in this code sample is for demo use and invalid!

String sLicensee = "Mem: Licensee String";    // AnsiString
long nBufSizeLic = sLicensee.WideCharBufSize();
long nLengthLic  = sLicensee.Length();
wchar_t*wcLicensee = new wchar_t[ nLengthLic + 1 ];

String sKey      = "License Key String";
long nBufSizeKey = sKey.WideCharBufSize();
long nLengthKey  = sKey.Length();
wchar_t*wcKey    = new wchar_t[ nLengthKey + 1 ];

    sLicensee.WideChar(wcLicensee, nBufSizeLic); // convert to widechar*
    sKey.WideChar(wcKey, nBufSizeKey);

    // apply the licence key at startup of your application
    pTC2->LicenseMe(wcLicensee, eLicKindDeveloper, 1, wcKey, eLicProdStd); // License the control (3=Developer, 7=ProdID)

    // to check if key was entered correctly (for debugging only):
    // pTC2->Licensing();

    delete []wcLicensee;  // always free memory and avoid memory leaks!
    delete []wcKey;
}

